package javax.swing;

import sun.misc.SharedSecrets;
import sun.swing.SwingUtilities2;
import java.security.AccessControlContext;
import sun.awt.AWTAccessor;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.misc.JavaSecurityAccess;
import sun.swing.UIAction;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragSourceContext;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DragGestureListener;
import java.awt.event.ActionEvent;
import java.awt.Toolkit;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.util.TooManyListenersException;
import javax.swing.event.EventListenerList;
import javax.swing.plaf.UIResource;
import java.awt.dnd.DropTarget;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import javax.swing.text.JTextComponent;
import sun.swing.SwingAccessor;
import sun.awt.SunToolkit;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.Component;
import sun.awt.AppContext;
import java.awt.dnd.DropTargetListener;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.awt.datatransfer.DataFlavor;
import java.lang.reflect.Method;
import java.beans.PropertyDescriptor;
import sun.reflect.misc.MethodUtil;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.Clipboard;
import java.awt.GraphicsEnvironment;
import java.awt.event.MouseEvent;
import java.awt.event.InputEvent;
import java.awt.Point;
import java.awt.Image;
import java.io.Serializable;

public class TransferHandler implements Serializable
{
    public static final int NONE = 0;
    public static final int COPY = 1;
    public static final int MOVE = 2;
    public static final int COPY_OR_MOVE = 3;
    public static final int LINK = 1073741824;
    private Image dragImage;
    private Point dragImageOffset;
    private String propertyName;
    private static TransferHandler.SwingDragGestureRecognizer recognizer;
    static final Action cutAction;
    static final Action copyAction;
    static final Action pasteAction;
    
    public static Action getCutAction() {
        return TransferHandler.cutAction;
    }
    
    public static Action getCopyAction() {
        return TransferHandler.copyAction;
    }
    
    public static Action getPasteAction() {
        return TransferHandler.pasteAction;
    }
    
    public TransferHandler(String propertyName) {
        this.propertyName = propertyName;
    }
    
    protected TransferHandler() {
        this(null);
    }
    
    public void setDragImage(Image dragImage) {
        this.dragImage = dragImage;
    }
    
    public Image getDragImage() {
        return this.dragImage;
    }
    
    public void setDragImageOffset(Point p) {
        this.dragImageOffset = new Point(p);
    }
    
    public Point getDragImageOffset() {
        if (this.dragImageOffset == null) {
            return new Point(0, 0);
        }
        return new Point(this.dragImageOffset);
    }
    
    public void exportAsDrag(JComponent component, InputEvent inputEvent, int n) {
        int sourceActions = this.getSourceActions(component);
        if (!(inputEvent instanceof MouseEvent) || (n != 1 && n != 2 && n != 1073741824) || (sourceActions & n) == 0x0) {
            n = 0;
        }
        if (n != 0 && !GraphicsEnvironment.isHeadless()) {
            if (TransferHandler.recognizer == null) {
                TransferHandler.recognizer = new TransferHandler.SwingDragGestureRecognizer(new TransferHandler.DragHandler());
            }
            TransferHandler.recognizer.gestured(component, (MouseEvent)inputEvent, sourceActions, n);
        }
        else {
            this.exportDone(component, null, 0);
        }
    }
    
    public void exportToClipboard(JComponent component, Clipboard clipboard, int n) throws IllegalStateException {
        if ((n == 1 || n == 2) && (this.getSourceActions(component) & n) != 0x0) {
            Transferable transferable = this.createTransferable(component);
            if (transferable != null) {
                try {
                    clipboard.setContents(transferable, null);
                    this.exportDone(component, transferable, n);
                    return;
                }
                catch (IllegalStateException ex) {
                    this.exportDone(component, transferable, 0);
                    throw ex;
                }
            }
        }
        this.exportDone(component, null, 0);
    }
    
    public boolean importData(TransferHandler.TransferSupport transferSupport) {
        return transferSupport.getComponent() instanceof JComponent && this.importData((JComponent)transferSupport.getComponent(), transferSupport.getTransferable());
    }
    
    public boolean importData(JComponent obj, Transferable transferable) {
        PropertyDescriptor propertyDescriptor = this.getPropertyDescriptor(obj);
        if (propertyDescriptor != null) {
            Method writeMethod = propertyDescriptor.getWriteMethod();
            if (writeMethod == null) {
                return false;
            }
            Class[] parameterTypes = writeMethod.getParameterTypes();
            if (parameterTypes.length != 1) {
                return false;
            }
            DataFlavor propertyDataFlavor = this.getPropertyDataFlavor(parameterTypes[0], transferable.getTransferDataFlavors());
            if (propertyDataFlavor != null) {
                try {
                    MethodUtil.invoke(writeMethod, obj, new Object[] { transferable.getTransferData(propertyDataFlavor) });
                    return true;
                }
                catch (Exception ex) {
                    System.err.println("Invocation failed");
                }
            }
        }
        return false;
    }
    
    public boolean canImport(TransferHandler.TransferSupport transferSupport) {
        return transferSupport.getComponent() instanceof JComponent && this.canImport((JComponent)transferSupport.getComponent(), transferSupport.getDataFlavors());
    }
    
    public boolean canImport(JComponent component, DataFlavor[] array) {
        PropertyDescriptor propertyDescriptor = this.getPropertyDescriptor(component);
        if (propertyDescriptor != null) {
            Method writeMethod = propertyDescriptor.getWriteMethod();
            if (writeMethod == null) {
                return false;
            }
            Class[] parameterTypes = writeMethod.getParameterTypes();
            if (parameterTypes.length != 1) {
                return false;
            }
            if (this.getPropertyDataFlavor(parameterTypes[0], array) != null) {
                return true;
            }
        }
        return false;
    }
    
    public int getSourceActions(JComponent component) {
        if (this.getPropertyDescriptor(component) != null) {
            return 1;
        }
        return 0;
    }
    
    public Icon getVisualRepresentation(Transferable transferable) {
        return null;
    }
    
    protected Transferable createTransferable(JComponent component) {
        PropertyDescriptor propertyDescriptor = this.getPropertyDescriptor(component);
        if (propertyDescriptor != null) {
            return new TransferHandler.PropertyTransferable(propertyDescriptor, component);
        }
        return null;
    }
    
    protected void exportDone(JComponent component, Transferable transferable, int n) {
    }
    
    private PropertyDescriptor getPropertyDescriptor(JComponent component) {
        if (this.propertyName == null) {
            return null;
        }
        Class<Object> class1 = component.getClass();
        BeanInfo beanInfo;
        try {
            beanInfo = Introspector.getBeanInfo(class1);
        }
        catch (IntrospectionException ex) {
            return null;
        }
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
        for (int i = 0; i < propertyDescriptors.length; ++i) {
            if (this.propertyName.equals(propertyDescriptors[i].getName())) {
                Method readMethod = propertyDescriptors[i].getReadMethod();
                if (readMethod != null) {
                    Class[] parameterTypes = readMethod.getParameterTypes();
                    if (parameterTypes == null || parameterTypes.length == 0) {
                        return propertyDescriptors[i];
                    }
                }
            }
        }
        return null;
    }
    
    private DataFlavor getPropertyDataFlavor(Class<?> clazz, DataFlavor[] array) {
        int i = 0;
        while (i < array.length) {
            DataFlavor dataFlavor = array[i];
            if ("application".equals(dataFlavor.getPrimaryType()) && "x-java-jvm-local-objectref".equals(dataFlavor.getSubType()) && clazz.isAssignableFrom(dataFlavor.getRepresentationClass())) {
                return dataFlavor;
            }
            ++i;
        }
        return null;
    }
    
    private static DropTargetListener getDropTargetListener() {
        synchronized (TransferHandler.DropHandler.class) {
            TransferHandler.DropHandler value = (TransferHandler.DropHandler)AppContext.getAppContext().get(TransferHandler.DropHandler.class);
            if (value == null) {
                value = new TransferHandler.DropHandler();
                AppContext.getAppContext().put(TransferHandler.DropHandler.class, value);
            }
            return value;
        }
    }
    
    static DropTargetListener access$200() {
        return getDropTargetListener();
    }
    
    static {
        TransferHandler.recognizer = null;
        cutAction = new TransferHandler.TransferAction("cut");
        copyAction = new TransferHandler.TransferAction("copy");
        pasteAction = new TransferHandler.TransferAction("paste");
    }
    
    public static class DropLocation
    {
        private final Point dropPoint;
        
        protected DropLocation(Point p) {
            if (p == null) {
                throw new IllegalArgumentException("Point cannot be null");
            }
            this.dropPoint = new Point(p);
            return;
        }
        
        public final Point getDropPoint() {
            return new Point(this.dropPoint);
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "[dropPoint=" + this.dropPoint + "]";
        }
    }
    
    public static final class TransferSupport
    {
        private boolean isDrop;
        private Component component;
        private boolean showDropLocationIsSet;
        private boolean showDropLocation;
        private int dropAction;
        private Object source;
        private TransferHandler.DropLocation dropLocation;
        
        private TransferSupport(Component component, DropTargetEvent dropTargetEvent) {
            this.dropAction = -1;
            this.isDrop = true;
            this.setDNDVariables(component, dropTargetEvent);
        }
        
        public TransferSupport(Component component, Transferable source) {
            this.dropAction = -1;
            if (component == null) {
                throw new NullPointerException("component is null");
            }
            if (source == null) {
                throw new NullPointerException("transferable is null");
            }
            this.isDrop = false;
            this.component = component;
            this.source = source;
            return;
        }
        
        private void setDNDVariables(Component component, DropTargetEvent source) {
            assert !(!this.isDrop);
            this.component = component;
            this.source = source;
            this.dropLocation = null;
            this.dropAction = -1;
            this.showDropLocationIsSet = false;
            if (this.source == null) {
                return;
            }
            assert !(!(this.source instanceof DropTargetDropEvent));
            Point point = (this.source instanceof DropTargetDragEvent) ? ((DropTargetDragEvent)this.source).getLocation() : ((DropTargetDropEvent)this.source).getLocation();
            if (SunToolkit.isInstanceOf(component, "javax.swing.text.JTextComponent")) {
                this.dropLocation = SwingAccessor.getJTextComponentAccessor().dropLocationForPoint((JTextComponent)component, point);
            }
            else if ((component instanceof JComponent)) {
                this.dropLocation = ((JComponent)component).dropLocationForPoint(point);
            }
            return;
        }
        
        public boolean isDrop() {
            return this.isDrop;
        }
        
        public Component getComponent() {
            return this.component;
        }
        
        private void assureIsDrop() {
            if (!this.isDrop) {
                throw new IllegalStateException("Not a drop");
            }
            return;
        }
        
        public TransferHandler.DropLocation getDropLocation() {
            this.assureIsDrop();
            if (this.dropLocation == null) {
                this.dropLocation = new TransferHandler.DropLocation((this.source instanceof DropTargetDragEvent) ? ((DropTargetDragEvent)this.source).getLocation() : ((DropTargetDropEvent)this.source).getLocation());
            }
            return this.dropLocation;
        }
        
        public void setShowDropLocation(boolean showDropLocation) {
            this.assureIsDrop();
            this.showDropLocation = showDropLocation;
            this.showDropLocationIsSet = true;
        }
        
        public void setDropAction(int n) {
            this.assureIsDrop();
            int n2 = n & this.getSourceDropActions();
            if (n2 != 1 && n2 != 2 && n2 != 1073741824) {
                throw new IllegalArgumentException("unsupported drop action: " + n);
            }
            this.dropAction = n;
            return;
        }
        
        public int getDropAction() {
            return (this.dropAction == -1) ? this.getUserDropAction() : this.dropAction;
        }
        
        public int getUserDropAction() {
            this.assureIsDrop();
            return (this.source instanceof DropTargetDragEvent) ? ((DropTargetDragEvent)this.source).getDropAction() : ((DropTargetDropEvent)this.source).getDropAction();
        }
        
        public int getSourceDropActions() {
            this.assureIsDrop();
            return (this.source instanceof DropTargetDragEvent) ? ((DropTargetDragEvent)this.source).getSourceActions() : ((DropTargetDropEvent)this.source).getSourceActions();
        }
        
        public DataFlavor[] getDataFlavors() {
            if (this.isDrop) {
                if ((this.source instanceof DropTargetDragEvent)) {
                    return ((DropTargetDragEvent)this.source).getCurrentDataFlavors();
                }
                return ((DropTargetDropEvent)this.source).getCurrentDataFlavors();
            }
            else {
                return ((Transferable)this.source).getTransferDataFlavors();
            }
        }
        
        public boolean isDataFlavorSupported(DataFlavor dataFlavor) {
            if (this.isDrop) {
                if ((this.source instanceof DropTargetDragEvent)) {
                    return ((DropTargetDragEvent)this.source).isDataFlavorSupported(dataFlavor);
                }
                return ((DropTargetDropEvent)this.source).isDataFlavorSupported(dataFlavor);
            }
            else {
                return ((Transferable)this.source).isDataFlavorSupported(dataFlavor);
            }
        }
        
        public Transferable getTransferable() {
            if (this.isDrop) {
                if ((this.source instanceof DropTargetDragEvent)) {
                    return ((DropTargetDragEvent)this.source).getTransferable();
                }
                return ((DropTargetDropEvent)this.source).getTransferable();
            }
            else {
                return (Transferable)this.source;
            }
        }
        
        static void access$500(TransferHandler.TransferSupport transferSupport, Component component, DropTargetEvent dropTargetEvent) {
            transferSupport.setDNDVariables(component, dropTargetEvent);
        }
    }
    
    static class PropertyTransferable implements Transferable
    {
        JComponent component;
        PropertyDescriptor property;
        
        PropertyTransferable(PropertyDescriptor property, JComponent component) {
            this.property = property;
            this.component = component;
        }
        
        @Override
        public DataFlavor[] getTransferDataFlavors() {
            DataFlavor[] array = { null };
            String string = "application/x-java-jvm-local-objectref;class=" + this.property.getPropertyType().getName();
            try {
                array[0] = new DataFlavor(string);
            }
            catch (ClassNotFoundException ex) {
                array = new DataFlavor[0];
            }
            return array;
        }
        
        @Override
        public boolean isDataFlavorSupported(DataFlavor dataFlavor) {
            Class<Object> propertyType = this.property.getPropertyType();
            return "application".equals(dataFlavor.getPrimaryType()) && "x-java-jvm-local-objectref".equals(dataFlavor.getSubType()) && dataFlavor.getRepresentationClass().isAssignableFrom(propertyType);
        }
        
        @Override
        public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
            if (!this.isDataFlavorSupported(flavor)) {
                throw new UnsupportedFlavorException(flavor);
            }
            Method readMethod = this.property.getReadMethod();
            Object invoke;
            try {
                invoke = MethodUtil.invoke(readMethod, this.component, (Object[])null);
            }
            catch (Exception ex) {
                throw new IOException("Property read failed: " + this.property.getName());
            }
            return invoke;
        }
    }
    
    static class SwingDropTarget extends DropTarget implements UIResource
    {
        private EventListenerList listenerList;
        
        SwingDropTarget(Component c) {
            super(c, 1073741827, null);
            try {
                super.addDropTargetListener(TransferHandler.access$200());
            }
            catch (TooManyListenersException ex) {}
        }
        
        @Override
        public void addDropTargetListener(DropTargetListener l) throws TooManyListenersException {
            if (this.listenerList == null) {
                this.listenerList = new EventListenerList();
            }
            this.listenerList.add(DropTargetListener.class, l);
        }
        
        @Override
        public void removeDropTargetListener(DropTargetListener l) {
            if (this.listenerList != null) {
                this.listenerList.remove(DropTargetListener.class, l);
            }
        }
        
        @Override
        public void dragEnter(DropTargetDragEvent dtde) {
            super.dragEnter(dtde);
            if (this.listenerList != null) {
                Object[] listenerList = this.listenerList.getListenerList();
                for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                    if (listenerList[i] == DropTargetListener.class) {
                        ((DropTargetListener)listenerList[i + 1]).dragEnter(dtde);
                    }
                }
            }
        }
        
        @Override
        public void dragOver(DropTargetDragEvent dtde) {
            super.dragOver(dtde);
            if (this.listenerList != null) {
                Object[] listenerList = this.listenerList.getListenerList();
                for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                    if (listenerList[i] == DropTargetListener.class) {
                        ((DropTargetListener)listenerList[i + 1]).dragOver(dtde);
                    }
                }
            }
        }
        
        @Override
        public void dragExit(DropTargetEvent dte) {
            super.dragExit(dte);
            if (this.listenerList != null) {
                Object[] listenerList = this.listenerList.getListenerList();
                for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                    if (listenerList[i] == DropTargetListener.class) {
                        ((DropTargetListener)listenerList[i + 1]).dragExit(dte);
                    }
                }
            }
            if (!this.isActive()) {
                DropTargetListener access$200 = TransferHandler.access$200();
                if (access$200 != null && access$200 instanceof TransferHandler.DropHandler) {
                    TransferHandler.DropHandler.access$300((TransferHandler.DropHandler)access$200, false);
                }
            }
        }
        
        @Override
        public void drop(DropTargetDropEvent dtde) {
            super.drop(dtde);
            if (this.listenerList != null) {
                Object[] listenerList = this.listenerList.getListenerList();
                for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                    if (listenerList[i] == DropTargetListener.class) {
                        ((DropTargetListener)listenerList[i + 1]).drop(dtde);
                    }
                }
            }
        }
        
        @Override
        public void dropActionChanged(DropTargetDragEvent dtde) {
            super.dropActionChanged(dtde);
            if (this.listenerList != null) {
                Object[] listenerList = this.listenerList.getListenerList();
                for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                    if (listenerList[i] == DropTargetListener.class) {
                        ((DropTargetListener)listenerList[i + 1]).dropActionChanged(dtde);
                    }
                }
            }
        }
    }
    
    private static class DropHandler implements DropTargetListener, Serializable, ActionListener
    {
        private Timer timer;
        private Point lastPosition;
        private Rectangle outer;
        private Rectangle inner;
        private int hysteresis;
        private Component component;
        private Object state;
        private TransferHandler.TransferSupport support;
        private static final int AUTOSCROLL_INSET = 10;
        
        private DropHandler() {
            this.outer = new Rectangle();
            this.inner = new Rectangle();
            this.hysteresis = 10;
            this.support = new TransferHandler.TransferSupport(null, (DropTargetEvent)null);
        }
        
        private void updateAutoscrollRegion(JComponent component) {
            Rectangle visibleRect = component.getVisibleRect();
            this.outer.setBounds(visibleRect.x, visibleRect.y, visibleRect.width, visibleRect.height);
            Insets insets = new Insets(0, 0, 0, 0);
            if ((component instanceof Scrollable)) {
                int n = 20;
                if (visibleRect.width >= n) {
                    Insets insets2 = insets;
                    int n2 = 10;
                    insets.right = n2;
                    insets2.left = n2;
                }
                if (visibleRect.height >= n) {
                    Insets insets3 = insets;
                    int n3 = 10;
                    insets.bottom = n3;
                    insets3.top = n3;
                }
            }
            this.inner.setBounds(visibleRect.x + insets.left, visibleRect.y + insets.top, visibleRect.width - (insets.left + insets.right), visibleRect.height - (insets.top + insets.bottom));
        }
        
        private void autoscroll(JComponent component, Point point) {
            if ((component instanceof Scrollable)) {
                Scrollable scrollable = (Scrollable)component;
                if (point.y < this.inner.y) {
                    int scrollableUnitIncrement = scrollable.getScrollableUnitIncrement(this.outer, 1, -1);
                    component.scrollRectToVisible(new Rectangle(this.inner.x, this.outer.y - scrollableUnitIncrement, this.inner.width, scrollableUnitIncrement));
                }
                else if (point.y > this.inner.y + this.inner.height) {
                    component.scrollRectToVisible(new Rectangle(this.inner.x, this.outer.y + this.outer.height, this.inner.width, scrollable.getScrollableUnitIncrement(this.outer, 1, 1)));
                }
                if (point.x < this.inner.x) {
                    int scrollableUnitIncrement2 = scrollable.getScrollableUnitIncrement(this.outer, 0, -1);
                    component.scrollRectToVisible(new Rectangle(this.outer.x - scrollableUnitIncrement2, this.inner.y, scrollableUnitIncrement2, this.inner.height));
                }
                else if (point.x > this.inner.x + this.inner.width) {
                    component.scrollRectToVisible(new Rectangle(this.outer.x + this.outer.width, this.inner.y, scrollable.getScrollableUnitIncrement(this.outer, 0, 1), this.inner.height));
                }
            }
        }
        
        private void initPropertiesIfNecessary() {
            if (this.timer == null) {
                Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
                Integer n = (Integer)defaultToolkit.getDesktopProperty("DnD.Autoscroll.interval");
                this.timer = new Timer((n == null) ? 100 : ((int)n), this);
                Integer n2 = (Integer)defaultToolkit.getDesktopProperty("DnD.Autoscroll.initialDelay");
                this.timer.setInitialDelay((n2 == null) ? 100 : ((int)n2));
                Integer n3 = (Integer)defaultToolkit.getDesktopProperty("DnD.Autoscroll.cursorHysteresis");
                if (n3 != null) {
                    this.hysteresis = (int)n3;
                }
            }
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            this.updateAutoscrollRegion((JComponent)this.component);
            if (this.outer.contains(this.lastPosition) && !this.inner.contains(this.lastPosition)) {
                this.autoscroll((JComponent)this.component, this.lastPosition);
            }
        }
        
        private void setComponentDropLocation(TransferHandler.TransferSupport transferSupport, boolean b) {
            TransferHandler.DropLocation dropLocation = (transferSupport == null) ? null : transferSupport.getDropLocation();
            if (SunToolkit.isInstanceOf(this.component, "javax.swing.text.JTextComponent")) {
                this.state = SwingAccessor.getJTextComponentAccessor().setDropLocation((JTextComponent)this.component, dropLocation, this.state, b);
            }
            else if ((this.component instanceof JComponent)) {
                this.state = ((JComponent)this.component).setDropLocation(dropLocation, this.state, b);
            }
        }
        
        private void handleDrag(DropTargetDragEvent dropTargetDragEvent) {
            TransferHandler transferHandler = ((TransferHandler.HasGetTransferHandler)this.component).getTransferHandler();
            if (transferHandler == null) {
                dropTargetDragEvent.rejectDrag();
                this.setComponentDropLocation(null, false);
                return;
            }
            TransferHandler.TransferSupport.access$500(this.support, this.component, dropTargetDragEvent);
            int canImport = transferHandler.canImport(this.support);
            if (canImport != 0) {
                dropTargetDragEvent.acceptDrag(this.support.getDropAction());
            }
            else {
                dropTargetDragEvent.rejectDrag();
            }
            canImport = (this.support.showDropLocationIsSet ? this.support.showDropLocation : canImport);
            this.setComponentDropLocation(canImport ? this.support : null, false);
            return;
        }
        
        @Override
        public void dragEnter(DropTargetDragEvent dropTargetDragEvent) {
            this.state = null;
            this.component = dropTargetDragEvent.getDropTargetContext().getComponent();
            this.handleDrag(dropTargetDragEvent);
            if ((this.component instanceof JComponent)) {
                this.lastPosition = dropTargetDragEvent.getLocation();
                this.updateAutoscrollRegion((JComponent)this.component);
                this.initPropertiesIfNecessary();
            }
        }
        
        @Override
        public void dragOver(DropTargetDragEvent dropTargetDragEvent) {
            this.handleDrag(dropTargetDragEvent);
            if (!(this.component instanceof JComponent)) {
                return;
            }
            Point location = dropTargetDragEvent.getLocation();
            if (Math.abs(location.x - this.lastPosition.x) > this.hysteresis || Math.abs(location.y - this.lastPosition.y) > this.hysteresis) {
                if (this.timer.isRunning()) {
                    this.timer.stop();
                }
            }
            else if (!this.timer.isRunning()) {
                this.timer.start();
            }
            this.lastPosition = location;
            return;
        }
        
        @Override
        public void dragExit(DropTargetEvent dropTargetEvent) {
            this.cleanup(false);
        }
        
        @Override
        public void drop(DropTargetDropEvent dropTargetDropEvent) {
            TransferHandler transferHandler = ((TransferHandler.HasGetTransferHandler)this.component).getTransferHandler();
            if (transferHandler == null) {
                dropTargetDropEvent.rejectDrop();
                this.cleanup(false);
                return;
            }
            TransferHandler.TransferSupport.access$500(this.support, this.component, dropTargetDropEvent);
            int canImport = transferHandler.canImport(this.support);
            if (canImport != 0) {
                dropTargetDropEvent.acceptDrop(this.support.getDropAction());
                canImport = (this.support.showDropLocationIsSet ? this.support.showDropLocation : canImport);
                this.setComponentDropLocation(canImport ? this.support : null, false);
                int importData;
                try {
                    importData = transferHandler.importData(this.support);
                }
                catch (RuntimeException ex) {
                    importData = false;
                }
                dropTargetDropEvent.dropComplete((boolean)importData);
                this.cleanup((boolean)importData);
            }
            else {
                dropTargetDropEvent.rejectDrop();
                this.cleanup(false);
            }
            return;
        }
        
        @Override
        public void dropActionChanged(DropTargetDragEvent dropTargetDragEvent) {
            if (this.component == null) {
                return;
            }
            this.handleDrag(dropTargetDragEvent);
            return;
        }
        
        private void cleanup(boolean b) {
            this.setComponentDropLocation(null, b);
            if ((this.component instanceof JComponent)) {
                ((JComponent)this.component).dndDone();
            }
            if (this.timer != null) {
                this.timer.stop();
            }
            this.state = null;
            this.component = null;
            this.lastPosition = null;
        }
        
        static void access$300(TransferHandler.DropHandler dropHandler, boolean b) {
            dropHandler.cleanup(b);
        }
    }
    
    private static class DragHandler implements DragGestureListener, DragSourceListener
    {
        private boolean scrolls;
        
        @Override
        public void dragGestureRecognized(DragGestureEvent dragGestureEvent) {
            JComponent component = (JComponent)dragGestureEvent.getComponent();
            TransferHandler transferHandler = component.getTransferHandler();
            Transferable transferable = transferHandler.createTransferable(component);
            if (transferable != null) {
                this.scrolls = component.getAutoscrolls();
                component.setAutoscrolls(false);
                try {
                    Image dragImage = transferHandler.getDragImage();
                    if (dragImage == null) {
                        dragGestureEvent.startDrag(null, transferable, this);
                    }
                    else {
                        dragGestureEvent.startDrag(null, dragImage, transferHandler.getDragImageOffset(), transferable, this);
                    }
                    return;
                }
                catch (RuntimeException ex) {
                    component.setAutoscrolls(this.scrolls);
                }
            }
            transferHandler.exportDone(component, transferable, 0);
        }
        
        @Override
        public void dragEnter(DragSourceDragEvent dragSourceDragEvent) {
        }
        
        @Override
        public void dragOver(DragSourceDragEvent dragSourceDragEvent) {
        }
        
        @Override
        public void dragExit(DragSourceEvent dragSourceEvent) {
        }
        
        @Override
        public void dragDropEnd(DragSourceDropEvent dragSourceDropEvent) {
            DragSourceContext dragSourceContext = dragSourceDropEvent.getDragSourceContext();
            JComponent component = (JComponent)dragSourceContext.getComponent();
            if (dragSourceDropEvent.getDropSuccess()) {
                component.getTransferHandler().exportDone(component, dragSourceContext.getTransferable(), dragSourceDropEvent.getDropAction());
            }
            else {
                component.getTransferHandler().exportDone(component, dragSourceContext.getTransferable(), 0);
            }
            component.setAutoscrolls(this.scrolls);
        }
        
        @Override
        public void dropActionChanged(DragSourceDragEvent dragSourceDragEvent) {
        }
    }
    
    private static class SwingDragGestureRecognizer extends DragGestureRecognizer
    {
        SwingDragGestureRecognizer(DragGestureListener dgl) {
            super(DragSource.getDefaultDragSource(), null, 0, dgl);
        }
        
        void gestured(JComponent component, MouseEvent awtie, int sourceActions, int dragAction) {
            this.setComponent(component);
            this.setSourceActions(sourceActions);
            this.appendEvent(awtie);
            this.fireDragGestureRecognized(dragAction, awtie.getPoint());
        }
        
        @Override
        protected void registerListeners() {
        }
        
        @Override
        protected void unregisterListeners() {
        }
    }
    
    static class TransferAction extends UIAction implements UIResource
    {
        private static final JavaSecurityAccess javaSecurityAccess;
        private static Object SandboxClipboardKey;
        
        TransferAction(String name) {
            super(name);
        }
        
        public boolean isEnabled(Object o) {
            return !(o instanceof JComponent) || ((JComponent)o).getTransferHandler() != null;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            Object source = actionEvent.getSource();
            PrivilegedAction<Void> privilegedAction = new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    TransferHandler.TransferAction.access$800(TransferHandler.TransferAction.this, actionEvent);
                    return null;
                }
            };
            AccessControlContext context = AccessController.getContext();
            AccessControlContext accessControlContext = AWTAccessor.getComponentAccessor().getAccessControlContext((Component)source);
            AccessControlContext accessControlContext2 = AWTAccessor.getAWTEventAccessor().getAccessControlContext(actionEvent);
            if (accessControlContext == null) {
                TransferHandler.TransferAction.javaSecurityAccess.doIntersectionPrivilege(privilegedAction, context, accessControlContext2);
            }
            else {
                TransferHandler.TransferAction.javaSecurityAccess.doIntersectionPrivilege(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        TransferHandler.TransferAction.javaSecurityAccess.doIntersectionPrivilege(privilegedAction, accessControlContext2);
                        return null;
                    }
                }, context, accessControlContext);
            }
        }
        
        private void actionPerformedImpl(ActionEvent actionEvent) {
            Object source = actionEvent.getSource();
            if ((source instanceof JComponent)) {
                JComponent component = (JComponent)source;
                TransferHandler transferHandler = component.getTransferHandler();
                Clipboard clipboard = this.getClipboard(component);
                String anObject = (String)this.getValue("Name");
                Transferable contents = null;
                try {
                    if (clipboard != null && transferHandler != null && anObject != null) {
                        if ("cut".equals(anObject)) {
                            transferHandler.exportToClipboard(component, clipboard, 2);
                        }
                        else if ("copy".equals(anObject)) {
                            transferHandler.exportToClipboard(component, clipboard, 1);
                        }
                        else if ("paste".equals(anObject)) {
                            contents = clipboard.getContents(null);
                        }
                    }
                }
                catch (IllegalStateException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(component);
                    return;
                }
                if (contents != null) {
                    transferHandler.importData(new TransferHandler.TransferSupport(component, contents));
                }
            }
        }
        
        private Clipboard getClipboard(JComponent component) {
            if (SwingUtilities2.canAccessSystemClipboard()) {
                return component.getToolkit().getSystemClipboard();
            }
            Clipboard value = (Clipboard)AppContext.getAppContext().get(TransferHandler.TransferAction.SandboxClipboardKey);
            if (value == null) {
                value = new Clipboard("Sandboxed Component Clipboard");
                AppContext.getAppContext().put(TransferHandler.TransferAction.SandboxClipboardKey, value);
                return value;
            }
            return value;
        }
        
        static void access$800(TransferHandler.TransferAction transferAction, ActionEvent actionEvent) {
            transferAction.actionPerformedImpl(actionEvent);
        }
        
        static {
            javaSecurityAccess = SharedSecrets.getJavaSecurityAccess();
            TransferHandler.TransferAction.SandboxClipboardKey = new Object();
        }
    }
    
    interface HasGetTransferHandler
    {
        public abstract TransferHandler getTransferHandler();
    }
}
